@@ -36,12 +36,6 @@ class FortniteException(Exception):
     pass
 
 
-class PurchaseException(FortniteException):
-    """This exception is raised if the game could not be purchased on
-    launch.
-    """
-
-
 class AuthException(FortniteException):
     """This exception is raised when auth fails by invalid credentials
     passed or some other misc failure.
